package com.asperasoft.samples.faspex.cmdclient;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

import org.apache.commons.lang.StringEscapeUtils;

/**
 * SAX parser that parses the XRDS returned by the faspex XRDS service and finds
 * the specified service.
 * 
 * @author Aspera
 */
class PackageParser extends DefaultHandler {
    private String currentElement = "";
    private boolean inEntry = false;
    private FaspexPackage currentPkg;
    private List<FaspexPackage> pkgList = new ArrayList<FaspexPackage>();
    private FaspexUser author;
    private FaspexUser currentRecipient;
    private StringBuilder characterData;
    
    @Override
    public void startElement(String uri, String localName, String qName,
            Attributes attributes) throws SAXException {
        // set the current element we're in
        currentElement = qName;
        characterData = new StringBuilder();

        if (currentElement.equalsIgnoreCase("entry")) {
            currentPkg = new FaspexPackage();
            inEntry = true;
        } else if (currentElement.equalsIgnoreCase("author"))
            author = new FaspexUser();
        else if (currentElement.equalsIgnoreCase("package:to"))
            currentRecipient = new FaspexUser();
        else if (inEntry && currentElement.equalsIgnoreCase("link")) {
            String faspURL = StringEscapeUtils.unescapeXml(StringEscapeUtils.unescapeXml(attributes.getValue("href")));
            currentPkg.addFaspURL(faspURL);
        }
    }

    @Override
    public void endElement(String uri, String localName, String qName)
            throws SAXException {
        if (qName.equalsIgnoreCase("entry")) {
            pkgList.add(currentPkg);
            inEntry = false;
        } else if (qName.equalsIgnoreCase("author")) {
            currentPkg.setAuthor(author);
        } else if (qName.equalsIgnoreCase("package:to")) {
            currentPkg.getRecipients().add(currentRecipient);
        }
        
        // no longer in an element
        currentElement = "";
    }

    @Override
    public void characters(char[] ch, int start, int length)
            throws SAXException {
        // need to buffer the character data because this method can be called
        // multiple times for a character sequence.
        String value = new String(ch, start, length).trim();
        characterData.append(value);
        value = characterData.toString();
        
        if (inEntry) {
            if (currentElement.equalsIgnoreCase("title"))
                currentPkg.setTitle(value);
            else if (currentElement.equalsIgnoreCase("content"))
                currentPkg.setContent(value);
            else if (currentElement.equalsIgnoreCase("id"))
                currentPkg.setId(value);
            else if (currentElement.equalsIgnoreCase("name"))
                author.setName(value);
            else if (currentElement.equalsIgnoreCase("email"))
                author.setEmail(value);
            else if (currentElement.equalsIgnoreCase("package:name"))
                currentRecipient.setName(value);
            else if (currentElement.equalsIgnoreCase("package:email"))
                currentRecipient.setEmail(value);
        }
    }
    
    /**
     * Parses the <code>InputStream</code> containing the Atom feed and builds
     * objects representing the packages contained in the feed.
     *     
     * @param is The <code>InputStream</code> containing the Atom feed
     * @return A list of packages parsed from the Atom feed
     */
    public List<FaspexPackage> getPackageList(InputStream is) 
            throws SAXException, ParserConfigurationException, IOException {
        SAXParserFactory factory = SAXParserFactory.newInstance();
        SAXParser parser = factory.newSAXParser();
        parser.parse(is, this);
        
        return pkgList;
    }
}
