/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.asperasoft.samples.faspex.cmdclient;

import java.io.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;


/**
 * SAX parser that parses the XRDS returned by the faspex XRDS service and finds
 * the specified service.
 * 
 * @author Aspera
 */
class ServiceParser extends DefaultHandler {
    private String currentElement = "";
    private StringBuilder serviceURI = new StringBuilder();
    private String serviceType;
    private boolean serviceFound = false;

    @Override
    public void startElement(String uri, String localName, String qName,
            Attributes attributes) throws SAXException {
        // set the current element we're in
        currentElement = qName;
    }

    @Override
    public void endElement(String uri, String localName, String qName)
            throws SAXException {
        // no longer in an element
        currentElement = "";
        
        if (qName.equalsIgnoreCase("URI")) {
            // we've finished parsing the URI, stop looking for more
            serviceFound = false;
        }
    }

    @Override
    public void characters(char[] ch, int start, int length) throws SAXException {
        String value = new String(ch, start, length).trim();
        if (currentElement.equalsIgnoreCase("Type")
                && value.equalsIgnoreCase(serviceType)) {
            // we've found the service we're looking for
            serviceFound = true;
        } else if (serviceFound && currentElement.equalsIgnoreCase("URI")) {
            // This method can be called multiple times on different chunks of text
            // Concatenate them to get the full URI.
            serviceURI.append(value);
        }
    }

    /**
     * Read an XRDS document and find the specified service if it exists.
     * 
     * @param is The <code>InputStream</code> containing the XRDS data.
     * @param serviceType The service type to look for.
     * @return The URI of the service.
     * @throws ServiceNotFoundException If no service matching <code>serviceType</code>
     * was found in the XRDS data.
     */
    public String findService(InputStream is, String serviceType)
            throws SAXException, ParserConfigurationException, IOException, ServiceNotFoundException {
        this.serviceType = serviceType;
        
        SAXParserFactory factory = SAXParserFactory.newInstance();
        SAXParser parser = factory.newSAXParser();
        parser.parse(is, this);

        // empty StringBuilder means no matches found
        if (serviceURI.toString().equals(""))
            throw new ServiceNotFoundException("No service matching service type was found.");

        return serviceURI.toString();
    }
}
